"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../../utils");
const fs_1 = require("fs");
const path_1 = require("path");
const errors_1 = require("../../errors");
/**
 * The default metadata transformer.
 *
 * If a metadata type doesn't have a transformer assigned to it, this one is used
 * during the conversion process. It leaves the component's metadata xml and source
 * files as-is.
 */
class DefaultTransformer {
    constructor(component) {
        this.component = component;
    }
    toMetadataFormat() {
        const result = { component: this.component, writeInfos: [] };
        let xmlDest = this.getRelativeDestination(this.component.xml);
        if (this.component.content) {
            for (const source of this.component.walkContent()) {
                result.writeInfos.push({
                    source: fs_1.createReadStream(source),
                    relativeDestination: this.getRelativeDestination(source)
                });
            }
        }
        else {
            xmlDest = xmlDest.slice(0, xmlDest.lastIndexOf(utils_1.META_XML_SUFFIX));
        }
        result.writeInfos.push({
            source: fs_1.createReadStream(this.component.xml),
            relativeDestination: xmlDest
        });
        return result;
    }
    toSourceFormat() {
        throw new errors_1.LibraryError('error_convert_not_implemented', ['source', this.component.type.name]);
    }
    getRelativeDestination(fsPath) {
        const { directoryName, suffix, inFolder } = this.component.type;
        // if there isn't a suffix, assume this is a mixed content component that must
        // reside in the directoryName of its type. trimUntil maintains the folder structure
        // the file resides in for the new destination.
        if (!suffix) {
            return this.trimUntil(fsPath, directoryName);
        }
        else if (inFolder) {
            const folderName = this.component.fullName.split('/')[0];
            return path_1.join(directoryName, folderName, path_1.basename(fsPath));
        }
        return path_1.join(directoryName, path_1.basename(fsPath));
    }
    trimUntil(fsPath, name) {
        const parts = fsPath.split(path_1.sep);
        const index = parts.findIndex(part => name === part);
        return parts.slice(index).join(path_1.sep);
    }
}
exports.DefaultTransformer = DefaultTransformer;
//# sourceMappingURL=default.js.map