"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const path_1 = require("path");
const _1 = require(".");
const transformers_1 = require("../convert/transformers");
const errors_1 = require("../errors");
const path_2 = require("../utils/path");
const registry_1 = require("../utils/registry");
const mixedContentSourceAdapter_1 = require("./adapters/mixedContentSourceAdapter");
const sourceAdapterFactory_1 = require("./adapters/sourceAdapterFactory");
const forceIgnore_1 = require("./forceIgnore");
/**
 * Resolver for metadata type and component objects.
 */
class RegistryAccess {
    /**
     * @param registry Custom registry data
     * @param tree `TreeContainer` to traverse with
     */
    constructor(registry, tree = new _1.NodeFSTreeContainer()) {
        this.registry = registry
            ? // deep freeze a copy, not the original object
                registry_1.deepFreeze(JSON.parse(JSON.stringify(registry)))
            : // registryData is already frozen
                _1.registryData;
        this.tree = tree;
        this.sourceAdapterFactory = new sourceAdapterFactory_1.SourceAdapterFactory(this.registry, tree);
        this.metadataTransformerFactory = new transformers_1.MetadataTransformerFactory(this.registry);
    }
    getApiVersion() {
        return this.registry.apiVersion;
    }
    /**
     * Get a metadata type definition.
     *
     * @param name Name of the metadata type
     */
    getTypeFromName(name) {
        const lower = name.toLowerCase().replace(/ /g, '');
        if (!this.registry.types[lower]) {
            throw new errors_1.TypeInferenceError('error_missing_type_definition', lower);
        }
        return this.registry.types[lower];
    }
    /**
     * Get the metadata component(s) from a file path.
     *
     * @param fsPath File path for a piece of metadata
     */
    getComponentsFromPath(fsPath) {
        if (!this.tree.exists(fsPath)) {
            throw new errors_1.TypeInferenceError('error_path_not_found', fsPath);
        }
        let pathForFetch = fsPath;
        this.forceIgnore = forceIgnore_1.ForceIgnore.findAndCreate(fsPath);
        if (this.tree.isDirectory(fsPath)) {
            // If we can determine a type from a directory path, and the end part of the path isn't
            // the directoryName of the type itself, we know the path is part of a mixedContent component
            const type = this.resolveType(fsPath);
            if (type) {
                const { directoryName, inFolder } = type;
                const parts = fsPath.split(path_1.sep);
                const folderOffset = inFolder ? 2 : 1;
                if (parts[parts.length - folderOffset] !== directoryName) {
                    pathForFetch =
                        mixedContentSourceAdapter_1.MixedContentSourceAdapter.findMetadataFromContent(fsPath, type, this.tree) || fsPath;
                }
            }
            if (pathForFetch === fsPath) {
                return this.getComponentsFromPathRecursive(fsPath);
            }
        }
        const component = this.resolveComponent(pathForFetch);
        return component ? [component] : [];
    }
    getTransformer(component) {
        return this.metadataTransformerFactory.getTransformer(component);
    }
    getComponentsFromPathRecursive(dir) {
        const dirQueue = [];
        const components = [];
        if (this.forceIgnore.denies(dir)) {
            return components;
        }
        for (const file of this.tree.readDirectory(dir)) {
            const fsPath = path_1.join(dir, file);
            if (this.tree.isDirectory(fsPath)) {
                dirQueue.push(fsPath);
            }
            else if (registry_1.parseMetadataXml(fsPath)) {
                const component = this.resolveComponent(fsPath);
                if (component) {
                    components.push(component);
                    // don't traverse further if not in a root type directory. performance optimization
                    // for mixed content types and ensures we don't add duplicates of the component.
                    const isMixedContent = !!this.registry.mixedContent[component.type.directoryName];
                    const typeDir = path_1.basename(path_1.dirname(component.type.inFolder ? path_1.dirname(fsPath) : fsPath));
                    if (isMixedContent && typeDir !== component.type.directoryName) {
                        return components;
                    }
                }
            }
        }
        for (const dir of dirQueue) {
            components.push(...this.getComponentsFromPathRecursive(dir));
        }
        return components;
    }
    resolveComponent(fsPath) {
        if (registry_1.parseMetadataXml(fsPath) && this.forceIgnore.denies(fsPath)) {
            // don't fetch the component if the metadata xml is denied
            return;
        }
        const type = this.resolveType(fsPath);
        if (type) {
            const adapter = this.sourceAdapterFactory.getAdapter(type, this.forceIgnore);
            return adapter.getComponent(fsPath);
        }
        throw new errors_1.TypeInferenceError('error_could_not_infer_type', fsPath);
    }
    resolveType(fsPath) {
        let typeId;
        // attempt 1 - check if the file is part of a mixed content type
        const pathParts = new Set(fsPath.split(path_1.sep));
        for (const directoryName of Object.keys(this.registry.mixedContent)) {
            if (pathParts.has(directoryName)) {
                typeId = this.registry.mixedContent[directoryName];
                // types with folders only have folder components living at the top level.
                // if the fsPath is a folder component, let a future strategy deal with it
                const isFolderType = this.getTypeFromName(typeId).inFolder;
                if (isFolderType && path_2.parentName(fsPath) === directoryName) {
                    typeId = undefined;
                }
                break;
            }
        }
        // attempt 2 - check if it's a metadata xml file
        if (!typeId) {
            const parsedMetaXml = registry_1.parseMetadataXml(fsPath);
            if (parsedMetaXml) {
                typeId = this.registry.suffixes[parsedMetaXml.suffix];
            }
        }
        // attempt 3 - try treating the file extension name as a suffix
        if (!typeId) {
            typeId = this.registry.suffixes[path_2.extName(fsPath)];
        }
        if (typeId) {
            return this.getTypeFromName(typeId);
        }
    }
}
exports.RegistryAccess = RegistryAccess;
//# sourceMappingURL=registryAccess.js.map