import { Connection } from '@salesforce/core';
import { SourcePath } from './common';
import { RegistryAccess, SourceComponent } from '../metadata-registry';
declare type CommonOptions = {
    /**
     * Set the max number of seconds to wait for the operation.
     */
    wait?: number;
    namespace?: string;
};
declare type CommonRetrieveOptions = {
    /**
     * Whether or not the files should be automatically converted to
     * [source format](https://developer.salesforce.com/docs/atlas.en-us.sfdx_dev.meta/sfdx_dev/sfdx_dev_source_file_format.htm)
     */
    convert?: boolean;
    /**
     * Whether or not existing source files should be overwritten.
     */
    overwrite?: boolean;
    /**
     * The directory to retrieve the components to.
     */
    output?: SourcePath;
};
declare type CommonPathOptions = {
    /**
     * Source paths of the files to perform the operation on.
     */
    paths: SourcePath[];
};
export declare type RetrieveOptions = CommonOptions & CommonRetrieveOptions & {
    components: SourceComponent[];
};
export declare type RetrievePathOptions = CommonOptions & CommonRetrieveOptions & CommonPathOptions;
export declare type ApiResult = {
    success: boolean;
    components: SourceComponent[];
    message?: string;
};
declare type WaitFlag = {
    wait?: number;
};
declare type NamespaceFlag = {
    namespace?: string;
};
export declare type MetadataApiDeployOptions = {
    allowMissingFiles?: boolean;
    autoUpdatePackage?: boolean;
    checkOnly?: boolean;
    ignoreWarnings?: boolean;
    performRetrieve?: boolean;
    purgeOnDelete?: boolean;
    rollbackOnError?: boolean;
    runAllTests?: boolean;
    runTests?: string[];
    singlePackage?: boolean;
};
export declare type MetadataDeployOptions = WaitFlag & {
    apiOptions?: MetadataApiDeployOptions;
};
export declare type ToolingDeployOptions = NamespaceFlag;
export declare type DeployPathOptions = CommonOptions & CommonPathOptions;
export declare type DeployResult = {
    State: DeployStatusEnum;
    ErrorMsg: string | null;
    isDeleted: boolean;
    DeployDetails: DeployDetails | null;
    outboundFiles?: string[];
    metadataFile: string;
};
export declare type DeployDetails = {
    componentFailures: SourceResult[];
    componentSuccesses: SourceResult[];
};
export declare type SourceResult = {
    columnNumber?: number;
    lineNumber?: number;
    problem?: string;
    problemType?: string;
    fileName?: string;
    fullName?: string;
    componentType: string;
    success?: boolean;
    changed: boolean;
    created: boolean;
    deleted: boolean;
};
export declare enum DeployStatusEnum {
    Completed = "Completed",
    Queued = "Queued",
    Error = "Error",
    Failed = "Failed"
}
export interface DeployRetrieveClient {
    /**
     * Retrieve metadata components and wait for the result.
     *
     * @param options Specify `components`, `output` and other optionals
     */
    retrieve(options: RetrieveOptions): Promise<ApiResult>;
    /**
     * Infer metadata components from source paths, retrieve them, and wait for the result.
     *
     * @param options Specify `paths`, `output` and other optionals
     */
    retrieveWithPaths(options: RetrievePathOptions): Promise<ApiResult>;
    /**
     * Deploy metadata components and wait for result.
     *
     * @param filePath Paths to source files to deploy
     */
    deploy(components: SourceComponent | SourceComponent[]): Promise<DeployResult>;
    /**
     * Infer metadata components from source path, deploy them, and wait for results.
     *
     * @param filePath Paths to source files to deploy
     */
    deployWithPaths(paths: SourcePath | SourcePath[]): Promise<DeployResult>;
}
export declare abstract class BaseApi implements DeployRetrieveClient {
    protected connection: Connection;
    protected registry: RegistryAccess;
    constructor(connection: Connection, registry: RegistryAccess);
    /**
     * @param options Specify `paths`, `output` and other optionals
     */
    abstract retrieveWithPaths(options: RetrievePathOptions): Promise<ApiResult>;
    abstract retrieve(options: RetrieveOptions): Promise<ApiResult>;
    abstract deploy(components: SourceComponent | SourceComponent[]): Promise<DeployResult>;
    abstract deployWithPaths(paths: SourcePath | SourcePath[]): Promise<DeployResult>;
}
export {};
